@@ -643,6 +643,27 @@ void odf_inc_d
                    a_hihilo,a_lohilo,a_hilolo,a_lololo);
 }
 
+void odf_dec
+ ( double *a_hihihi, double *a_lohihi, double *a_hilohi, double *a_lolohi,
+   double *a_hihilo, double *a_lohilo, double *a_hilolo, double *a_lololo,
+   double b_hihihi, double b_lohihi, double b_hilohi, double b_lolohi,
+   double b_hihilo, double b_lohilo, double b_hilolo, double b_lololo )
+{
+   double mbhihihi = -b_hihihi;
+   double mblohihi = -b_lohihi;
+   double mbhilohi = -b_hilohi;
+   double mblolohi = -b_lolohi;
+   double mbhihilo = -b_hihilo;
+   double mblohilo = -b_lohilo;
+   double mbhilolo = -b_hilolo;
+   double mblololo = -b_lololo;
+
+   odf_inc(a_hihihi,a_lohihi,a_hilohi,a_lolohi,
+           a_hihilo,a_lohilo,a_hilolo,a_lololo,
+           mbhihihi,mblohihi,mbhilohi,mblolohi,
+           mbhihilo,mblohilo,mbhilolo,mblololo);
+}
+
 void odf_minus
  ( double *a_hihihi, double *a_lohihi, double *a_hilohi, double *a_lolohi,
    double *a_hihilo, double *a_lohilo, double *a_hilolo, double *a_lololo )
@@ -1341,6 +1362,29 @@ void odf_mul_od_d
                    c_hihilo,c_lohilo,c_hilolo,c_lololo);
 }
 
+void odf_mlt_d
+ ( double *a_hihihi, double *a_lohihi, double *a_hilohi, double *a_lolohi,
+   double *a_hihilo, double *a_lohilo, double *a_hilolo, double *a_lololo,
+   double b )
+{
+   double c_hihihi,c_lohihi,c_hilohi,c_lolohi;
+   double c_hihilo,c_lohilo,c_hilolo,c_lololo;
+
+   odf_mul_od_d(*a_hihihi,*a_lohihi,*a_hilohi,*a_lolohi,
+                *a_hihilo,*a_lohilo,*a_hilolo,*a_lololo,b,
+                &c_hihihi,&c_lohihi,&c_hilohi,&c_lolohi,
+                &c_hihilo,&c_lohilo,&c_hilolo,&c_lololo);
+
+   *a_hihihi = c_hihihi;
+   *a_lohihi = c_lohihi;
+   *a_hilohi = c_hilohi;
+   *a_lolohi = c_lolohi;
+   *a_hihilo = c_hihilo;
+   *a_lohilo = c_lohilo;
+   *a_hilolo = c_hilolo;
+   *a_lololo = c_lololo;
+}
+
 void odf_div
  ( double a_hihihi, double a_lohihi, double a_hilohi, double a_lolohi,
    double a_hihilo, double a_lohilo, double a_hilolo, double a_lololo,
