@@ -16,7 +16,7 @@
 #include "write_gpu_timings.h"
 #include "dbl2_polynomials_host.h"
 #include "dbl2_polynomials_kernels.h"
-#include "test_helpers.h"
+#include "job_makers.h"
 #include "dbl2_polynomials_testers.h"
 
 using namespace std;
@@ -40,9 +40,9 @@ int main_dbl2_test_polynomial
    }
    if(vrblvl > 0) cout << "  Seed used : " << seedused << endl;
 
-   double realsum = test_dbl2_real_polynomial
+   double realsum = test_dbl2_polynomial
                        (dim,nbr,nva,pwr,deg,vrblvl-1,jobrep,mode);
-   double compsum = test_dbl2_complex_polynomial
+   double compsum = test_cmplx2_polynomial
                        (dim,nbr,nva,pwr,deg,vrblvl-1,jobrep,mode);
 
    int fail = int(realsum > tol) + int(compsum > tol);
@@ -75,7 +75,7 @@ int main_dbl2_test_polynomial
    return fail;
 }
 
-void dbl2_make_input
+int dbl2_make_input
  ( int dim, int nbr, int nva, int pwr, int deg,
    int *nvr, int **idx, int **exp,
    double **inputhi, double **inputlo,
@@ -95,7 +95,12 @@ void dbl2_make_input
             cout << inputhi[i][j] << "  " << inputlo[i][j] << endl;
       }
    }
-   if(nva == 0) // random supports
+   if(nva < 0) // read supports
+   {
+      read_supports(dim,nbr,nvr);
+      for(int i=0; i<nbr; i++) idx[i] = new int[nvr[i]];
+   }
+   else if(nva == 0) // random supports
    {
       make_supports(dim,nbr,nvr);
       for(int i=0; i<nbr; i++) idx[i] = new int[nvr[i]];
@@ -145,17 +150,21 @@ void dbl2_make_input
             cout << cffhi[i][j] << "  " << cfflo[i][j] << endl;
       }
    }
-   if(nva == 0)
+   if(nva <= 0)
    {
       bool dup = duplicate_supports(dim,nbr,nvr,idx,verbose);
       if(dup)
-         cout << "Duplicate supports found." << endl;
+      {
+         cout << "Duplicates in support found." << endl;
+         return 1;
+      }
       else if(verbose)
-         cout << "No duplicate supports found." << endl;
+         cout << "No duplicates in support found." << endl;
    }
+   return 0;
 }
 
-void cmplx2_make_input
+int cmplx2_make_input
  ( int dim, int nbr, int nva, int pwr, int deg,
    int *nvr, int **idx, int **exp,
    double **inputrehi, double **inputrelo,
@@ -179,7 +188,12 @@ void cmplx2_make_input
                  << inputimhi[i][j] << "  " << inputimlo[i][j] << endl;
       }
    }
-   if(nva == 0) // random supports
+   if(nva < 0) // read supports
+   {
+      read_supports(dim,nbr,nvr);
+      for(int i=0; i<nbr; i++) idx[i] = new int[nvr[i]];
+   }
+   else if(nva == 0) // random supports
    {
       make_supports(dim,nbr,nvr);
       for(int i=0; i<nbr; i++) idx[i] = new int[nvr[i]];
@@ -237,14 +251,65 @@ void cmplx2_make_input
                  << cffimhi[i][j] << "  " << cffimlo[i][j] << endl;
       }
    }
-   if(nva == 0)
+   if(nva <= 0)
    {
       bool dup = duplicate_supports(dim,nbr,nvr,idx,verbose);
       if(dup)
-         cout << "Duplicate supports found." << endl;
+      {
+         cout << "Duplicates in support found." << endl;
+         return 1;
+      }
       else if(verbose)
-         cout << "No duplicate supports found." << endl;
+         cout << "No duplicates in support found." << endl;
    }
+   return 0;
+}
+
+double dbl2_error_sum1
+ ( int dim, int deg,
+   double **resultshi_h, double **resultslo_h,
+   double **resultshi_d, double **resultslo_d, bool verbose )
+
+{
+   double err = 0.0;
+
+   if(verbose) cout << "The value of the polynomial :" << endl;
+   for(int i=0; i<=deg; i++)
+   {
+      if(verbose)
+      {
+         cout << resultshi_h[dim][i] << "  "
+              << resultslo_h[dim][i] << endl;
+         cout << resultshi_d[dim][i] << "  "
+              << resultslo_d[dim][i] << endl;
+      }
+      err = err + abs(resultshi_h[dim][i] - resultshi_d[dim][i])
+                + abs(resultslo_h[dim][i] - resultslo_d[dim][i]);
+   }
+   if(verbose) cout << "error : " << err << endl;
+
+   double sumerr = err;
+
+   for(int k=0; k<dim; k++)
+   {
+      if(verbose) cout << "Derivative " << k << " :" << endl;
+      err = 0.0;
+      for(int i=0; i<=deg; i++)
+      {
+         if(verbose)
+         {
+            cout << resultshi_h[k][i] << "  "
+                 << resultslo_h[k][i] << endl;
+            cout << resultshi_d[k][i] << "  "
+                 << resultslo_d[k][i] << endl;
+         }
+         err = err + abs(resultshi_h[k][i] - resultshi_d[k][i])
+                   + abs(resultslo_h[k][i] - resultslo_d[k][i]);
+      }
+      if(verbose) cout << "error : " << err << endl;
+      sumerr = sumerr + err;
+   }
+   return sumerr;
 }
 
 double dbl2_error_sum
@@ -302,6 +367,66 @@ double dbl2_error_sum
    return sumerr;
 }
 
+double cmplx2_error_sum1
+ ( int dim, int deg,
+   double **resultsrehi_h, double **resultsrelo_h,
+   double **resultsimhi_h, double **resultsimlo_h,
+   double **resultsrehi_d, double **resultsrelo_d,
+   double **resultsimhi_d, double **resultsimlo_d, bool verbose )
+{
+   double err = 0.0;
+
+   if(verbose) cout << "The value of the polynomial :" << endl;
+   for(int i=0; i<=deg; i++)
+   {
+      if(verbose)
+      {
+         cout << resultsrehi_h[dim][i] << "  "
+              << resultsrelo_h[dim][i] << endl;
+         cout << resultsimhi_h[dim][i] << "  "
+              << resultsimlo_h[dim][i] << endl;
+         cout << resultsrehi_d[dim][i] << "  "
+              << resultsrelo_d[dim][i] << endl;
+         cout << resultsimhi_d[dim][i] << "  "
+              << resultsimlo_d[dim][i] << endl;
+      }
+      err = err + abs(resultsrehi_h[dim][i] - resultsrehi_d[dim][i])
+                + abs(resultsrelo_h[dim][i] - resultsrelo_d[dim][i])
+                + abs(resultsimhi_h[dim][i] - resultsimhi_d[dim][i])
+                + abs(resultsimlo_h[dim][i] - resultsimlo_d[dim][i]);
+   }
+   if(verbose) cout << "error : " << err << endl;
+
+   double sumerr = err;
+
+   for(int k=0; k<dim; k++)
+   {
+      if(verbose) cout << "Derivative " << k << " :" << endl;
+      err = 0.0;
+      for(int i=0; i<=deg; i++)
+      {
+         if(verbose)
+         {
+            cout << resultsrehi_h[k][i] << "  "
+                 << resultsrelo_h[k][i] << endl;
+            cout << resultsimhi_h[k][i] << "  "
+                 << resultsimlo_h[k][i] << endl;
+            cout << resultsrehi_d[k][i] << "  "
+                 << resultsrelo_d[k][i] << endl;
+            cout << resultsimhi_d[k][i] << "  "
+                 << resultsimlo_d[k][i] << endl;
+         }
+         err = err + abs(resultsrehi_h[k][i] - resultsrehi_d[k][i])
+                   + abs(resultsrelo_h[k][i] - resultsrelo_d[k][i])
+                   + abs(resultsimhi_h[k][i] - resultsimhi_d[k][i])
+                   + abs(resultsimlo_h[k][i] - resultsimlo_d[k][i]);
+      }
+      if(verbose) cout << "error : " << err << endl;
+      sumerr = sumerr + err;
+   }
+   return sumerr;
+}
+
 double cmplx2_error_sum
  ( int dim, int deg,
    double **results1rehi_h, double **results1relo_h,
@@ -380,10 +505,12 @@ double cmplx2_error_sum
    return sumerr;
 }
 
-double test_dbl2_real_polynomial
+double test_dbl2_polynomial
  ( int dim, int nbr, int nva, int pwr, int deg, int verbose, bool jobrep,
    int mode )
 {
+   cout << "*** testing the real arithmetic on real data ***" << endl;
+
    if(nbr < 1)
       return 0.0;
    else
@@ -419,10 +546,13 @@ double test_dbl2_real_polynomial
 
       bool vrb = (verbose > 1);
 
-      dbl2_make_input(dim,nbr,nva,pwr,deg,nvr,idx,exp,
-                      inputhi,inputlo,csthi,cstlo,cffhi,cfflo,vrb);
-
-
+      int fail = dbl2_make_input(dim,nbr,nva,pwr,deg,nvr,idx,exp,inputhi,
+                                 inputlo,csthi,cstlo,cffhi,cfflo,vrb);
+      if(fail == 1)
+      {
+          cout << "Duplicates in support, returning 0 as error." << endl;
+          return 0.0;
+      }
       ConvolutionJobs cnvjobs(dim);
       AdditionJobs addjobs(dim,nbr);
 
@@ -452,9 +582,11 @@ double test_dbl2_real_polynomial
       }
       double sumerr = 0.0;
       if(mode == 2)
+      {
          sumerr = dbl2_error_sum(dim,deg,output1hi_h,output1lo_h,
                      output2hi_h,output2lo_h,outputhi_d,outputlo_d,vrb);
-
+         cout << "sum of all errors " << sumerr << endl;
+      }
       if(verbose > 0)
       {
          if(jobrep) write_jobs_report(dim,nva,nbr,deg,cnvjobs,addjobs);
@@ -467,10 +599,12 @@ double test_dbl2_real_polynomial
    }
 }
 
-double test_dbl2_complex_polynomial
+double test_cmplx2_polynomial
  ( int dim, int nbr, int nva, int pwr, int deg, int verbose, bool jobrep,
    int mode )
 {
+   cout << "*** testing the complex arithmetic on complex data ***" << endl;
+
    if(nbr < 1)
       return 0.0;
    else
@@ -538,16 +672,27 @@ double test_dbl2_complex_polynomial
 
       bool vrb = (verbose > 1);
 
-      cmplx2_make_input(dim,nbr,nva,pwr,deg,nvr,idx,exp,
-                        inputrehi,inputrelo,inputimhi,inputimlo,
-                        cstrehi,cstrelo,cstimhi,cstimlo,
-                        cffrehi,cffrelo,cffimhi,cffimlo,vrb);
-
+      int fail = cmplx2_make_input(dim,nbr,nva,pwr,deg,nvr,idx,exp,
+                                   inputrehi,inputrelo,inputimhi,inputimlo,
+                                   cstrehi,cstrelo,cstimhi,cstimlo,
+                                   cffrehi,cffrelo,cffimhi,cffimlo,vrb);
+      if(fail == 1)
+      {
+          cout << "Duplicates in support, returning 0 as error." << endl;
+          return 0.0;
+      }
       ConvolutionJobs cnvjobs(dim);
       AdditionJobs addjobs(dim,nbr);
 
       make_all_jobs(dim,nbr,nvr,idx,&cnvjobs,&addjobs,vrb);
 
+      ComplexConvolutionJobs cmplxcnvjobs(dim);
+      ComplexIncrementJobs cmplxincjobs(cmplxcnvjobs,vrb);
+      ComplexAdditionJobs cmplxaddjobs(dim,nbr);
+
+      make_all_complex_jobs
+         (dim,nbr,nvr,idx,&cmplxcnvjobs,&cmplxincjobs,&cmplxaddjobs,vrb);
+
       double timelapsec1_h,timelapsec2_h;
       double cnvlapms,addlapms,timelapms_d,walltimes_d;
 
@@ -571,6 +716,7 @@ double test_dbl2_complex_polynomial
       if((mode == 0) || (mode == 2))
       {
          if(vrb) cout << "Computing on the device ..." << endl;
+/*
          GPU_cmplx2_poly_evaldiff
             (deg+1,dim,nbr,deg,nvr,idx,
              cstrehi,cstrelo,cstimhi,cstimlo,
@@ -579,14 +725,24 @@ double test_dbl2_complex_polynomial
              outputrehi_d,outputrelo_d,outputimhi_d,outputimlo_d,
              cnvjobs,addjobs,&cnvlapms,&addlapms,&timelapms_d,
              &walltimes_d,vrb);
+ */
+         GPU_cmplx2vectorized_poly_evaldiff
+            (deg+1,dim,nbr,deg,nvr,idx,
+             cstrehi,cstrelo,cstimhi,cstimlo,cffrehi,cffrelo,cffimhi,cffimlo,
+             inputrehi,inputrelo,inputimhi,inputimlo,
+             outputrehi_d,outputrelo_d,outputimhi_d,outputimlo_d,
+             cmplxcnvjobs,cmplxincjobs,cmplxaddjobs,
+             &cnvlapms,&addlapms,&timelapms_d,&walltimes_d,vrb);
       }
       double sumerr = 0.0;
       if(mode == 2)
+      {
          sumerr = cmplx2_error_sum(dim,deg,
                      output1rehi_h,output1relo_h,output1imhi_h,output1imlo_h,
                      output2rehi_h,output2relo_h,output2imhi_h,output2imlo_h,
                      outputrehi_d,outputrelo_d,outputimhi_d,outputimlo_d,vrb);
-
+         cout << "sum of all errors " << sumerr << endl;
+      }
       if(verbose > 0)
       {
          if(jobrep) write_jobs_report(dim,nva,nbr,deg,cnvjobs,addjobs);
