@@ -14,7 +14,7 @@
 #include "addition_jobs.h"
 #include "write_job_counts.h"
 #include "write_gpu_timings.h"
-#include "test_helpers.h"
+#include "job_makers.h"
 #include "dbl_polynomials_host.h"
 #include "dbl_polynomials_kernels.h"
 #include "dbl_polynomials_testers.h"
@@ -40,9 +40,9 @@ int main_dbl_test_polynomial
    }
    if(vrblvl > 0) cout << "  Seed used : " << seedused << endl;
 
-   double realsum = test_dbl_real_polynomial
+   double realsum = test_dbl_polynomial
                        (dim,nbr,nva,pwr,deg,vrblvl-1,jobrep,mode);
-   double compsum = test_dbl_complex_polynomial
+   double compsum = test_cmplx_polynomial
                        (dim,nbr,nva,pwr,deg,vrblvl-1,jobrep,mode);
 
    int fail = int(realsum > tol) + int(compsum > tol);
@@ -75,7 +75,7 @@ int main_dbl_test_polynomial
    return fail;
 }
 
-void dbl_make_input
+int dbl_make_input
  ( int dim, int nbr, int nva, int pwr, int deg,
    int *nvr, int **idx, int **exp,
    double **input, double *cst, double **cff, bool verbose )
@@ -92,7 +92,12 @@ void dbl_make_input
          for(int j=0; j<=deg; j++) cout << input[i][j] << endl;
       }
    }
-   if(nva == 0) // random supports
+   if(nva < 0) // read supports
+   {
+      read_supports(dim,nbr,nvr);
+      for(int i=0; i<nbr; i++) idx[i] = new int[nvr[i]];
+   }
+   else if(nva == 0) // random supports
    {
       make_supports(dim,nbr,nvr);
       for(int i=0; i<nbr; i++) idx[i] = new int[nvr[i]];
@@ -139,17 +144,21 @@ void dbl_make_input
          for(int j=0; j<=deg; j++) cout << " " << cff[i][j] << endl;
       }
    }
-   if(nva == 0)
+   if(nva <= 0)
    {
       bool dup = duplicate_supports(dim,nbr,nvr,idx,verbose);
       if(dup)
-         cout << "Duplicate supports found." << endl;
+      {
+         cout << "Duplicates in support found." << endl;
+         return 1;
+      }
       else if(verbose)
-         cout << "No duplicate supports found." << endl;
+         cout << "No duplicates in support found." << endl;
    }
+   return 0;
 }
 
-void cmplx_make_input
+int cmplx_make_input
  ( int dim, int nbr, int nva, int pwr, int deg,
    int *nvr, int **idx, int **exp,
    double **inputre, double **inputim, double *cstre, double *cstim,
@@ -168,7 +177,12 @@ void cmplx_make_input
             cout << inputre[i][j] << "  " << inputim[i][j] << endl;
       }
    }
-   if(nva == 0) // random supports
+   if(nva < 0) // read supports
+   {
+      read_supports(dim,nbr,nvr);
+      for(int i=0; i<nbr; i++) idx[i] = new int[nvr[i]];
+   }
+   else if(nva == 0) // random supports
    {
       make_supports(dim,nbr,nvr);
       for(int i=0; i<nbr; i++) idx[i] = new int[nvr[i]];
@@ -218,14 +232,56 @@ void cmplx_make_input
             cout << cffre[i][j] << "  " << cffim[i][j] << endl;
       }
    }
-   if(nva == 0)
+   if(nva <= 0)
    {
       bool dup = duplicate_supports(dim,nbr,nvr,idx,verbose);
       if(dup)
-         cout << "Duplicate supports found." << endl;
+      {
+         cout << "Duplicates in support found." << endl;
+         return 1;
+      }
       else if(verbose)
-         cout << "No duplicate supports found." << endl;
+         cout << "No duplicates in support found." << endl;
    }
+   return 0;
+}
+
+double dbl_error_sum1
+ ( int dim, int deg, double **results_h, double **results_d, bool verbose )
+{
+   double err = 0.0;
+
+   if(verbose) cout << "The value of the polynomial :" << endl;
+   for(int i=0; i<=deg; i++)
+   {
+      if(verbose)
+      {
+         cout << results_h[dim][i] << endl;
+         cout << results_d[dim][i] << endl;
+      }
+      err = err + abs(results_h[dim][i] - results_d[dim][i]);
+   }
+   if(verbose) cout << "error : " << err << endl;
+
+   double sumerr = err;
+
+   for(int k=0; k<dim; k++)
+   {
+      if(verbose) cout << "Derivative " << k << " :" << endl;
+      err = 0.0;
+      for(int i=0; i<=deg; i++)
+      {
+         if(verbose)
+         {
+            cout << results_h[k][i] << endl;
+            cout << results_d[k][i] << endl;
+         }
+         err = err + abs(results_h[k][i] - results_d[k][i]);
+      }
+      if(verbose) cout << "error : " << err << endl;
+      sumerr = sumerr + err;
+   }
+   return sumerr;
 }
 
 double dbl_error_sum
@@ -271,6 +327,52 @@ double dbl_error_sum
    return sumerr;
 }
 
+double cmplx_error_sum1
+ ( int dim, int deg,
+   double **resultsre_h, double **resultsim_h,
+   double **resultsre_d, double **resultsim_d, bool verbose )
+{
+   double err = 0.0;
+
+   if(verbose) cout << "The value of the polynomial :" << endl;
+   for(int i=0; i<=deg; i++)
+   {
+      if(verbose)
+      {
+         cout << resultsre_h[dim][i] << "  "
+              << resultsim_h[dim][i] << endl;
+         cout << resultsre_d[dim][i] <<  "  "
+              << resultsim_d[dim][i] << endl;
+      }
+      err = err + abs(resultsre_h[dim][i] - resultsre_d[dim][i])
+                + abs(resultsim_h[dim][i] - resultsim_d[dim][i]);
+   }
+   if(verbose) cout << "error : " << err << endl;
+
+   double sumerr = err;
+
+   for(int k=0; k<dim; k++)
+   {
+      if(verbose) cout << "Derivative " << k << " :" << endl;
+      err = 0.0;
+      for(int i=0; i<=deg; i++)
+      {
+         if(verbose)
+         {
+            cout << resultsre_h[k][i] << "  "
+                 << resultsim_h[k][i] << endl;
+            cout << resultsre_d[k][i] << "  "
+                 << resultsim_d[k][i] << endl;
+         }
+         err = err + abs(resultsre_h[k][i] - resultsre_d[k][i])
+                   + abs(resultsim_h[k][i] - resultsim_d[k][i]);
+      }
+      if(verbose) cout << "error : " << err << endl;
+      sumerr = sumerr + err;
+   }
+   return sumerr;
+}
+
 double cmplx_error_sum
  ( int dim, int deg,
    double **results1re_h, double **results1im_h,
@@ -326,10 +428,12 @@ double cmplx_error_sum
    return sumerr;
 }
 
-double test_dbl_real_polynomial
+double test_dbl_polynomial
  ( int dim, int nbr, int nva, int pwr, int deg, int verbose, bool jobrep,
    int mode )
 {
+   cout << "*** testing the real arithmetic on real data ***" << endl;
+
    if(nbr < 1)
       return 0.0;
    else
@@ -354,8 +458,13 @@ double test_dbl_real_polynomial
 
       const bool vrb = (verbose > 1);
 
-      dbl_make_input(dim,nbr,nva,pwr,deg,nvr,idx,exp,input,cst,cff,vrb);
-
+      int fail = dbl_make_input(dim,nbr,nva,pwr,deg,nvr,idx,exp,
+                                input,cst,cff,vrb);
+      if(fail == 1)
+      {
+          cout << "Duplicates in support, returning 0 as error." << endl;
+          return 0.0;
+      }
       ConvolutionJobs cnvjobs(dim);
       AdditionJobs addjobs(dim,nbr);
 
@@ -384,8 +493,10 @@ double test_dbl_real_polynomial
       }
       double sumerr = 0.0;
       if(mode == 2)
+      {
          sumerr = dbl_error_sum(dim,deg,output1_h,output2_h,output_d,vrb);
-
+         cout << "sum of all errors " << sumerr << endl;
+      }
       if(verbose > 0)
       {
          if(jobrep) write_jobs_report(dim,nva,nbr,deg,cnvjobs,addjobs);
@@ -398,10 +509,12 @@ double test_dbl_real_polynomial
    }
 }
 
-double test_dbl_complex_polynomial
+double test_cmplx_polynomial
  ( int dim, int nbr, int nva, int pwr, int deg, int verbose, bool jobrep,
    int mode )
 {
+   cout << "*** testing the complex arithmetic on complex data ***" << endl;
+
    if(nbr < 1)
       return 0.0;
    else
@@ -451,14 +564,25 @@ double test_dbl_complex_polynomial
 
       const bool vrb = (verbose > 1);
 
-      cmplx_make_input(dim,nbr,nva,pwr,deg,nvr,idx,exp,inputre,inputim,
-                       cstre,cstim,cffre,cffim,vrb);
-
+      int fail = cmplx_make_input(dim,nbr,nva,pwr,deg,nvr,idx,exp,inputre,
+                                  inputim,cstre,cstim,cffre,cffim,vrb);
+      if(fail == 1)
+      {
+          cout << "Duplicates in support, returning 0 as error." << endl;
+          return 0.0;
+      }
       ConvolutionJobs cnvjobs(dim);
       AdditionJobs addjobs(dim,nbr);
 
       make_all_jobs(dim,nbr,nvr,idx,&cnvjobs,&addjobs,vrb);
 
+      ComplexConvolutionJobs cmplxcnvjobs(dim);
+      ComplexIncrementJobs cmplxincjobs(cmplxcnvjobs,vrb);
+      ComplexAdditionJobs cmplxaddjobs(dim,nbr);
+
+      make_all_complex_jobs
+         (dim,nbr,nvr,idx,&cmplxcnvjobs,&cmplxincjobs,&cmplxaddjobs,vrb);
+
       double timelapsec1_h,timelapsec2_h;
       double cnvlapms,addlapms,timelapms_d,walltimes_d;
 
@@ -476,16 +600,25 @@ double test_dbl_complex_polynomial
       if((mode == 0) || (mode == 2))
       {
          if(vrb) cout << "Computing on the device ..." << endl;
+/*
          GPU_cmplx_poly_evaldiff
             (deg+1,dim,nbr,deg,nvr,idx,cstre,cstim,cffre,cffim,
              inputre,inputim,outputre_d,outputim_d,cnvjobs,addjobs,
              &cnvlapms,&addlapms,&timelapms_d,&walltimes_d,vrb);
+ */
+         GPU_cmplxvectorized_poly_evaldiff
+            (deg+1,dim,nbr,deg,nvr,idx,cstre,cstim,cffre,cffim,
+             inputre,inputim,outputre_d,outputim_d,
+             cmplxcnvjobs,cmplxincjobs,cmplxaddjobs,
+             &cnvlapms,&addlapms,&timelapms_d,&walltimes_d,vrb);
       }
       double sumerr = 0.0;
       if(mode == 2)
+      {
          sumerr = cmplx_error_sum(dim,deg,output1re_h,output1im_h,
                      output2re_h,output2im_h,outputre_d,outputim_d,vrb);
-
+         cout << "sum of all errors " << sumerr << endl;
+      }
       if(verbose > 0)
       {
          if(jobrep) write_jobs_report(dim,nva,nbr,deg,cnvjobs,addjobs);
