@@ -1,14 +1,14 @@
 # standalone test on computing the mixed volume
-include("JuPHC.jl")
+include("PHCpack.jl")
 verbose = 10
 polynomials = ["x^3 + 2*x*y - 1;", "x + y - 1;"]
-JuPHC.setDoubleSystem(2, polynomials, verbose)
-pols = JuPHC.getDoubleSystem(verbose)
+PHCpack.setDoubleSystem(2, polynomials, verbose)
+pols = PHCpack.getDoubleSystem(verbose)
 println("the polynomials :")
 for pol in pols
     println(pol)
 end
-mv = JuPHC.mixedVolume(true, verbose)
+mv = PHCpack.mixedVolume(true, verbose)
 println("the mixed volume : ", mv)
-mv = JuPHC.mixedVolume(false, verbose)
+mv = PHCpack.mixedVolume(false, verbose)
 println("the mixed volume : ", mv)
